/*
 *  GEM MINE - The ROCK Linux Package Manager
 *  Copyright (C) 2002 Clifford Wolf
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <signal.h>

#include "mine.h"

enum {
        MINE_MODE_NONE,
        MINE_MODE_INSTALL,
        MINE_MODE_REMOVE,
        MINE_MODE_PKGLIST,
        MINE_MODE_SHOWFILE,
        MINE_MODE_SHOWKEY
};

int mine_mode = MINE_MODE_NONE;
int mine_mode_test;
int mine_mode_verbose;
int mine_mode_force;
int mine_mode_header_line;
int mine_mode_header_block;

char * mine_file_key_name;
char * mine_root_dir;

void help() {
	fprintf(stderr,
"\n"
"GEM MINE v" MINE_VERSION
" - ROCK Linux Package Management Tool, (C) 2002 Clifford Wolf\n"
"\n"
"   This program is free software; you can redistribute it and/or modify\n"
"   it under the terms of the GNU General Public License as published by\n"
"   the Free Software Foundation; either version 2 of the License, or\n"
"   (at your option) any later version. This program is distributed in the\n"
"   hope that it will be useful, but WITHOUT ANY WARRANTY.\n"
"\n"
"Unkown option, missing option or not allowed combination of options!\n"
"\n"
"    mine -i [ -t ] [ -v ] [ -f ] [ -R root ] package1.gem [ .. ]\n"
"    mine -r [ -t ] [ -v ] [ -f ] [ -R root ] package1 [ package2 [ .. ] ]\n"
"\n"
"    mine -{qplmcd} [ -h | -H ] [ package1 [ package2.gem [ .. ] ] ]\n"
"    mine -k key    [ -h | -H ] package1.gem [ package2.gem [ .. ] ]\n"
"\n"
"    mine -C var-adm-dir package-tar-bz2 package-name gem-file\n"
"    mine -T var-adm-dir root-dir package-name package-tar-bz2\n"
"\n"
"Hint: After installing, updating or removing packages you should run the\n"
"      script 'cron.run'. This updates some importand system files.\n"
"\n"
	);
	exit(1);
}

void set_mode(int mode) {
	if (mine_mode == MINE_MODE_NONE) mine_mode = mode;
	else help();
}

int main(int argc, char ** argv) {
	int opt;

	/* No Zombies */
	signal(SIGCHLD, SIG_IGN);

	if ( argc == 6 && !strcmp(argv[1], "-C") ) {
		gem_create(argv[2], argv[3], argv[4], argv[5]);
		return 0;
	}

	if ( argc == 6 && !strcmp(argv[1], "-T") ) {
		printf("( cd %s\ncut -f2- -d' ' %s/flists/%s | \\\n"
		       "\ttar -cf- --no-recursion --files-from=- | bzip2\n"
		       ") > %s\n", argv[3][0] ? argv[3] : "/",
		       argv[2], argv[4], argv[5]);
		return 0;
	}

	while ( (opt = getopt(argc, argv,
		              "irqplmcdk:tvfhHR:")) != -1 ) {
		switch (opt) {
			case 'i':
				set_mode(MINE_MODE_INSTALL);
				break;
			case 'r':
				set_mode(MINE_MODE_REMOVE);
				break;

			case 'q':
				set_mode(MINE_MODE_PKGLIST);
				break;
			case 'p':
				set_mode(MINE_MODE_SHOWFILE);
				mine_file_key_name = "packages";
				break;
			case 'l':
				set_mode(MINE_MODE_SHOWFILE);
				mine_file_key_name = "flists";
				break;
			case 'm':
				set_mode(MINE_MODE_SHOWFILE);
				mine_file_key_name = "md5sums";
				break;
			case 'c':
				set_mode(MINE_MODE_SHOWFILE);
				mine_file_key_name = "cksums";
				break;
			case 'd':
				set_mode(MINE_MODE_SHOWFILE);
				mine_file_key_name = "dependencies";
				break;

			case 'k':
				set_mode(MINE_MODE_SHOWKEY);
				mine_file_key_name = optarg;
				break;

			case 't':
				if ( !mine_mode_test &&
				     (mine_mode == MINE_MODE_INSTALL ||
				      mine_mode == MINE_MODE_REMOVE) )
				{
					mine_mode_test=1;
				} else
					help();
				break;
			case 'v':
				if ( !mine_mode_verbose &&
				     (mine_mode == MINE_MODE_INSTALL ||
				      mine_mode == MINE_MODE_REMOVE) )
				{
					mine_mode_verbose=1;
				} else
					help();
				break;

			case 'f':
				if ( !mine_mode_force &&
				     (mine_mode == MINE_MODE_INSTALL ||
				      mine_mode == MINE_MODE_REMOVE) )
				{
					mine_mode_force=1;
				} else
					help();
				break;

			case 'h':
				if ( !mine_mode_header_line &&
				     !mine_mode_header_block &&
				     (mine_mode == MINE_MODE_SHOWFILE ||
				      mine_mode == MINE_MODE_SHOWKEY) )
				{
					mine_mode_header_line=1;
				} else
					help();
				break;
			case 'H':
				if ( !mine_mode_header_line &&
				     !mine_mode_header_block &&
				     (mine_mode == MINE_MODE_SHOWFILE ||
				      mine_mode == MINE_MODE_SHOWKEY) )
				{
					mine_mode_header_block=1;
				} else
					help();
				break;

			case 'R':
				if ( mine_root_dir == NULL &&
				     (mine_mode == MINE_MODE_INSTALL ||
				      mine_mode == MINE_MODE_REMOVE) )
				{
					mine_root_dir = optarg;
				} else
					help();
				break;

			default:
				help();
		}
	}

	if ( mine_mode == MINE_MODE_NONE ) help();
	if ( mine_root_dir == NULL ) mine_root_dir = "";

	if (optind < argc)
	{
	  while (optind < argc) {
		switch (mine_mode) {
			case MINE_MODE_INSTALL:
				gem_install(mine_root_dir, mine_mode_test,
				            mine_mode_verbose, mine_mode_force,
				            argv[optind]);
				break;

			case MINE_MODE_REMOVE:
				gem_remove(mine_root_dir, mine_mode_test,
				           mine_mode_verbose, mine_mode_force,
				           argv[optind]);
				break;

			case MINE_MODE_PKGLIST:
				gem_pkglist(argv[optind]);
				break;

			case MINE_MODE_SHOWFILE:
				gem_showfile( mine_file_key_name,
					      mine_mode_header_block,
					      argv[optind]);
				break;

			case MINE_MODE_SHOWKEY:
				gem_showkey( mine_file_key_name,
					     mine_mode_header_line,
					     mine_mode_header_block,
					     argv[optind]);
				break;

			default:
				help();
		}
		optind++;;
	  }
	}
	else
	{
		struct dirent **namelist;
		int n;

		if ( mine_mode == MINE_MODE_INSTALL ||
		     mine_mode == MINE_MODE_REMOVE  ||
		     mine_mode == MINE_MODE_SHOWKEY ) help();

		n = scandir("/var/adm/packages", &namelist, 0, alphasort);
		if (n < 0) {
			perror("Can't read /var/adm/packages");
			exit(1);
		}
		while(n--) {
			switch (mine_mode) {
				case MINE_MODE_PKGLIST:
					gem_pkglist(namelist[n]->d_name);
					break;

				case MINE_MODE_SHOWFILE:
					gem_showfile( mine_file_key_name,
					              mine_mode_header_block,
					              namelist[n]->d_name);
					break;
			}
			free(namelist[n]);
		}
		free(namelist);
	}

        return 0;
}

