/*
 *  GEM MINE - The ROCK Linux Package Manager
 *  Copyright (C) 2002 Clifford Wolf
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

#include "mine.h"
#include "md5sum.h"

int gem_remove(char * root, int mode_test, int mode_verbose, 
               int mode_force, char * package)
{
	char buffer[1024];
	char realfn[1024];
	char *filename;
	int errors = 0;
	FILE *f;

	if ( ! mode_force )
		md5sum_initdb(root, mode_verbose);

	snprintf(buffer, 1024, "%s/var/adm/flists/%s", root, package);
	f = fopen(buffer, "r");
	if ( f == NULL ) {
		fprintf(stderr, "No such package: %s\n", package);
		return 1;
	}

	if ( mode_test && mode_verbose )
		printf("-- %s --\n", package);

	while ( fgets(buffer, 1024, f) != NULL )
	{
		strtok(buffer, " \t\n");
		filename = strtok(NULL, " \t\n");

		if ( ! mode_force && md5sum_check(root, filename) ) {
			if ( ! mode_test || ! mode_verbose )
				printf("%s: ", package);
			printf("WARNING: Skip modified/duplicate "
			       "file: %s\n", filename);
		}
		else
		if ( mode_test ) {
			if ( ! mode_verbose )
				printf("%s: ", package);
			printf("removing %s\n", filename);
		}
		else
		{
			if ( mode_verbose )
				printf("%s: removing %s\n", package, filename);
			snprintf(realfn, 1024, "%s/%s", root, filename);
			if ( remove(realfn) ) {
				printf("While removing package %s: %s: %s\n",
				       package, realfn, strerror(errno));
				errors++;
			}
		}
	}

	fclose(f);

	if ( errors )
		fprintf(stderr, "%d error%s while removing package %s.\n",
		                errors, errors != 1 ? "s" : "", package);
	return errors == 0;
}

